<?php

namespace Drupal\give\Entity;

use Drupal\give\Plugin\Field\FieldType\PaymentMethod;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\give\DonationInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\give\GiveStripeInterface;
use Drupal\user\EntityOwnerTrait;
use Drupal\user\EntityOwnerInterface;
use Drupal\Core\Entity\EntityChangedTrait;

/**
 * Defines the give donation entity.
 *
 * @ContentEntityType(
 *   id = "give_donation",
 *   label = @Translation("Give donation"),
 *   handlers = {
 *     "access" = "Drupal\give\GiveDonationAccessControlHandler",
 *     "view_builder" = "Drupal\give\DonationViewBuilder",
 *     "list_builder" = "Drupal\give\Form\Donation\DonationListBuilder",
 *     "views_data" = "Drupal\give\DonationViewsData",
 *     "form" = {
 *       "default" = "Drupal\give\Form\Donation\DonationForm",
 *       "add" = "Drupal\give\Form\Donation\PaymentForm",
 *       "edit" = "Drupal\give\Form\Donation\DonationEditForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *       "complete" = "Drupal\give\Form\Donation\DonationCompleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\give\Entity\DonationRouteProvider"
 *     }
 *   },
 *   base_table = "give_donation",
 *   admin_permission = "administer give",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "give_form",
 *     "uuid" = "uuid",
 *     "langcode" = "langcode",
 *     "owner" = "uid"
 *   },
 *   links = {
 *     "canonical" = "/donation/{give_donation}",
 *     "add-form" = "/donation/add",
 *     "delete-form" = "/donation/{give_donation}/delete",
 *     "complete-form" = "/donation/{give_donation}/complete",
 *     "collection" = "/admin/reports/donation"
 *   },
 *   bundle_entity_type = "give_form",
 *   field_ui_base_route = "entity.give_form.edit_form",
 * )
 *
 * @todo Change the method field from an integer to the method's name.
 * @todo Change the completed field to status, which is inherited from ContentEntityInterface
 * @todo remove most of the get/set functions here
 */
class Donation extends ContentEntityBase implements DonationInterface, EntityOwnerInterface {

  use EntityOwnerTrait;
  use EntityChangedTrait;

  const NOT_RECURRING = -1;

  /**
   * {@inheritdoc}
   */
  public function getGiveForm() {
    return $this->get('give_form')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    // We always build the donation's label from the donor's name and e-mail,
    // the amount of the donation, and the subject field if present.
    $label = $this->getGiveForm()->get('label') . ': '. $this->getDonorName();
    if ($mail = $this->mail->value) {
      $label .= " ($mail)";
    }
    return $label;
  }

  /**
   * {@inheritdoc}
   */
  public function getDonorName() {
    if ($this->uid->target_id) {
      if ($users = $this->uid->referencedEntities()) {
        return reset($users)->label();
      }
    }
    return $this->name->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormattedAmount() {
    return format_stripe_currency($this->amount->value);
  }

  /**
   * {@inheritdoc}
   */
  public function recurring() {
    return $this->recurring->value != static::NOT_RECURRING;
  }

  /**
   * {@inheritdoc}
   *
   * @note Seems this function is not used
   */
  public function setRecurrenceIntervalUnit($interval) {
    if ($interval != 'month') {
      throw new \Exception(t("Unsupported interval %interval. Interval periods other than month-based are not currently supported.", ['%interval' => $interval]));
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRecurrenceIntervalUnit() {
    $give_form = $this->getGiveForm();
    $frequencies = $give_form->getFrequencies();
    return $frequencies[$this->get('recurring')->value]['interval'];
  }

  /**
   * {@inheritdoc}
   */
  public function getRecurrenceIntervalCount() {
    $give_form = $this->getGiveForm();
    $frequencies = $give_form->getFrequencies();
    return $frequencies[$this->get('recurring')->value]['interval_count'];
  }

  /**
   * {@inheritdoc}
   *
   * @todo Maybe a higher level function handles the NOT_RECURRING option,
   * something like what happens with the field cardinality constants.
   *
   * @note Seems this function is not used
   */
  public function setRecurrenceIntervalCount($count) {
    $this->set('recurring', $count);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRecurrence() : string {
    $give_form = $this->getGiveForm();
    $frequencies = $give_form->getFrequencies();
    $value = $this->recurring->value;
    if ($value == NOT_RECURRING) {
      return 'No';
    }
    // Minus 1 means NOT_RECURRING
    if (isset($frequencies[$value -1])) {
      return $frequencies[$value -1]['description'];
    }
    return 'No';
  }

  /**
   * {@inheritdoc}
   *
   * Should really validate input here against PaymentMethod constants.
   */
  public function setMethod($method) {
    $this->set('method', $method);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getReplyType() {
    if ($this->method->value == PaymentMethod::GIVE_VIA_STRIPE) {
      if ($this->recurring()) {
        return 'recurring';
      }
      else {
        return 'onetime';
      }
    }

    if ($this->method->value == PaymentMethod::GIVE_VIA_CHECK) {
      return 'pledge';
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getRecurringProductName() {
    if (!$this->recurring()) {
      throw new \Exception('Cannot get recurring product name on non recurring product.');
    }
    $give_form = $this->getGiveForm();
    $frequencies = $give_form->getFrequencies();
    return $this->getFormattedAmount() . ' ' . $frequencies[$this->recurring->value]['description'];

  }

  /**
   * {@inheritdoc}
   */
  public function getStatementDescriptor() {
    // TODO allow setting of organization name in Give config and only using
    // sitename as a fallback.
    $config_system_site = \Drupal::config('system.site');
    $short_sitename = trim(substr($config_system_site->get('name'), 0, 12));
    $descriptor = $this->recurring->value ?
      $this->getRecurringProductName() :
      $this->label;
    return strtoupper(substr($short_sitename . ' ' . $descriptor, 0, 22));
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getStripeToken() {
    return $this->get('stripe_token')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setStripeToken($token) {
    $this->set('stripe_token', $token);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCardLast4() {
    return $this->get('card_last4')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCardLast4($last4) {
    $this->set('card_last4', $last4);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCardBrand() {
    return $this->get('card_brand')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCardBrand($brand) {
    $this->set('card_brand', $brand);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCardFunding() {
    return $this->get('card_funding')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCardFunding($funding) {
    $this->set('card_funding', $funding);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressLine1() {
    $this->get('address_line1');
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressLine1($line) {
    $this->set('address_line1', $line);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressLine2() {
    $this->get('address_line2');
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressLine2($line) {
    $this->set('address_line2', $line);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressCity() {
    $this->get('address_city');
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressCity($city) {
    $this->set('address_city', $city);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressState() {
    $this->get('address_state');
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressState($state) {
    $this->set('address_state', $state);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressZip() {
    $this->get('address_zip');
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressZip($zip) {
    $this->set('address_zip', $zip);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAddressCountry() {
    $this->get('address_country');
  }

  /**
   * {@inheritdoc}
   */
  public function setAddressCountry($country) {
    $this->set('address_country', $country);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isCompleted() {
    return (bool) $this->get('complete')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCompleted($completed = TRUE) {
    $this->set('complete', $completed ? DONATION_COMPLETED : DONATION_NOT_COMPLETED);
    return $this;
  }

  /**
   * Set card info.
   *
   * Helper function to set card last four, brand, and funding source from a
   * GiveStripe entity.
   *
   * @param \Drupal\give\GiveStripeInterface $give_stripe
   *   The donation.
   */
  public function setCardInfo(GiveStripeInterface $give_stripe) {
    $charge = $give_stripe->charge;
    $funding = $charge->source->funding;
    $brand = $charge->source->brand;
    $last4 = $charge->source->last4;
    $this->setCardFunding($funding);
    $this->setCardBrand($brand);
    $return = $this->setCardLast4($last4);
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields += static::ownerBaseFieldDefinitions($entity_type);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t("The donor's name"))
      ->setDescription(t('The name of the person that is sending the give donation.'))
      ->setDisplayConfigurable('view', TRUE);

    $fields['amount'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Donation amount'))
      ->setDescription('The amount of the donation, in cents.')
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    // Currently this is just a string but it should probably be an enumerated field extended by a hook_alter
    $fields['method'] = BaseFieldDefinition::create('give_method')
      ->setLabel(t('Method'))
      ->setDescription(t('The donation method (payment card, check pledge).'))
      ->setDisplayConfigurable('view', TRUE);

    $fields['mail'] = BaseFieldDefinition::create('email')
      ->setLabel(t("The donor's email"))
      ->setDescription(t('The email of the person that is sending the give donation.'))
      ->setDisplayConfigurable('view', TRUE);

    $fields['recurring'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Recurring'))
      ->setDescription(t('The interval counts (in number of months) at which the donation should recur, or negative one if not recurring.'))
      ->setDisplayConfigurable('view', TRUE);

    $fields['telephone'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Phone'))
      ->setDescription(t('The telephone number of the donor.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 20);

    $fields['check_or_other_information'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Further information'))
      ->setDescription(t('Any questions or explain anything needed to arrange for giving donation.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 2000);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the donation was last edited.'))
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the donation was created.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setTranslatable(TRUE)
      ->setReadOnly(TRUE);

    $fields['address_line1'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Address line 1'))
      ->setDescription(t('The street address or PO Box of the donor; used in billing address.'))
      ->setDisplayConfigurable('view', TRUE);

    $fields['address_line2'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Address line 2'))
      ->setDescription(t('Optional apartment/suite/unit of the donor; used in billing address.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 100);

    $fields['address_city'] = BaseFieldDefinition::create('string')
      ->setLabel(t('City or district'))
      ->setDescription(t('The town of the donor; used in billing address.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 100);

    $fields['address_zip'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Postal code'))
      ->setDescription(t('ZIP or postal code of the donor; used in billing address.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 100);

    $fields['address_state'] = BaseFieldDefinition::create('string')
      ->setLabel(t('State or province'))
      ->setDescription(t('The state/province/region of the donor; used in billing address.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 100);

    $fields['address_country'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Country'))
      ->setDescription(t('The country the donor; used in billing address.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 100);

    $fields['complete'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Completed donation'))
      ->setDisplayConfigurable('view', TRUE)
      ->setDefaultValue(FALSE);

    // Does this need to be stored? Maybe for recurring payments?
    $fields['stripe_token'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Stripe token'))
      ->setDescription(t('The token returned by Stripe used to tell Stripe to process the donation.'))
      ->setSetting('max_length', 56);

    $fields['card_brand'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Card brand'))
      ->setDescription(t('The card brand (Visa, MasterCard, etc).'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 30);

    $fields['card_funding'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Card funding'))
      ->setDescription(t('The card funding type (credit, debit).'))
      ->setDisplayConfigurable('view', TRUE)
      ->setSetting('max_length', 30);

    $fields['card_last4'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Last four'))
      ->setDescription(t('The last four digits of the credit/debit card, if applicable.'))
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

}
